unit PictPlus; // Version 0.90


// PictPlus - Picture Plus Property Editor
// Freeware Property Editor

// This code is provided as is with no warranty.  I will make additions
// and corrections as time permits.  Feel free to drop me a line if
// you want something added.

// New versions will be posted on the web at the
// Tangent Technologies, Inc WEB page at
// www.mindspring.com/~tti

// Have Fun!
// Nick Hustak
// President, Tangent Technologies, Inc
// TTI@mindspring.com
// www.mindspring.com/~tti
// 770-427-2118
// CSERVE: 72134,123
// President, Atlanta Delphi Developers Group
// www.mindspring.com/~addg

interface

uses
  WinTypes, Classes, Controls, DsgnIntf, WinProcs, Graphics, Dialogs, Forms,
  Buttons, TypInfo, SysUtils, FileCtrl, ExtCtrls, StdCtrls, Registry, Mask,
  ComCtrls;

type
  TPropertyType = ( ptBitmap, ptIcon, ptPicture );
  TFileType = ( ftBitmap, ftIcon, ftMetafile );

  TfrmPictPlus = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    cbxFile: TFileListBox;
    imgView: TImage;
    Panel3: TPanel;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    pnlBottom: TPanel;
    cbxHistory: TComboBox;
    btnTest: TBitBtn;
    btnClear: TBitBtn;
    Panel4: TPanel;
    cbxDir: TDirectoryListBox;
    cbxDrive: TDriveComboBox;
    btnAbout: TBitBtn;
    chbEnableDisable: TCheckBox;
    pnlAbout: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    lsvImages: TListView;
    btnView: TBitBtn;
    iltImages: TImageList;
    btnSave: TBitBtn;
    procedure cbxFileClick(Sender: TObject);
    procedure cbxFileDblClick(Sender: TObject);
    procedure cbxHistoryClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbxHistoryExit(Sender: TObject);
    procedure cbxHistoryDropDown(Sender: TObject);
    procedure cbxDirChange(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure imgViewClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure chbEnableDisableClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnViewClick(Sender: TObject);
    procedure lsvImagesClick(Sender: TObject);
    procedure lsvImagesDblClick(Sender: TObject);
  private
    { Private declarations }
    OldHistoryWidth: Integer;
    Ini: TRegIniFile;
    procedure LoadListView;    
  public
    { Public declarations }
    PropertyType: TPropertyType;
    FileType: TFileType;
    Cleared: Boolean;
  end;

  TPicturePropertyPlus = class( TClassProperty )
    private
    public
      function  GetValue: String;override;
      procedure SetValue( const Value:String );override;
      procedure Edit;override;
      function  GetAttributes: TPropertyAttributes;override;
      procedure SetFileType( const Filename: String );
    end;

procedure Register;


implementation

{$R *.DFM}

var
  frmPictPlus: TfrmPictPlus;

procedure TPicturePropertyPlus.Edit;
var
  Picture: TPicture;
  ImageName: String;
  FrontLoad: TPicture;
  EnumType: PTypeInfo;
begin
Picture := TPicture.Create;
EnumType := GetPropType;
frmPictPlus := TfrmPictPlus.Create( Application );
if UpperCase( EnumType.Name ) = 'TBITMAP' then
  frmPictPlus.PropertyType := ptBitmap
else if UpperCase( ENumType.Name ) = 'TICON' then
  frmPictPlus.PropertyType := ptIcon
else if UpperCase( ENumType.Name ) = 'TPICTURE' then
  frmPictPlus.PropertyType := ptPicture;

case frmPictPlus.PropertyType of
  ptBitmap: frmPictPlus.cbxFile.Mask := '*.BMP';
  ptIcon: frmPictPlus.cbxFile.Mask := '*.ICO';
  ptPicture: frmPictPlus.cbxFile.Mask := '*.BMP;*.ICO;*.EMF;*.WMF';
  end;

frmPictPlus.Cleared := False;
case frmPictPlus.PropertyType of
  ptBitmap:
    begin
    FrontLoad := TPicture.Create;
    FrontLoad.Bitmap := TBitMap( GetOrdProp( GetComponent( 0 ), GetPropInfo ) );
    if not FrontLoad.Bitmap.Empty then
      begin
      frmPictPlus.imgView.Picture.Assign( FrontLoad );
      //btnSave.Enabled := True;
      end;
    FrontLoad.Free;
    end;
  ptIcon:
    begin
    FrontLoad := TPicture.Create;
    FrontLoad.Icon := TIcon( GetOrdProp( GetComponent( 0 ), GetPropInfo ) );
    if not FrontLoad.Icon.Empty then
      begin
      frmPictPlus.imgView.Picture.Assign( FrontLoad );
      //btnSave.Enabled := True;
      end;
    FrontLoad.Free;
    end;
  ptPicture:
    begin
    FrontLoad := TPicture( GetOrdProp( GetComponent( 0 ), GetPropInfo ) );
    if Picture <> nil then
      begin
      //imgView.Picture.Assign( FrontLoad );
      //frmPictPlus.btnSave.Enabled := True;
      end;
    end;
  end;
frmPictPlus.ShowModal;
if frmPictPlus.ModalResult = mrOK then
  begin
  if frmPictPlus.lsvImages.Visible then
    ImageName := frmPictPlus.cbxDir.Directory + '/' + frmPictPlus.lsvImages.Selected.Caption
  else
    ImageName := frmPictPlus.cbxFile.Filename;
  Picture.LoadFromFile( ImageName );
  case frmPictPlus.PropertyType of
    ptBitmap: SetOrdProp( GetComponent( 0 ), GetPropInfo, LongInt( Picture.Bitmap ) );
    ptIcon: SetOrdProp( GetComponent( 0 ), GetPropInfo, LongInt( Picture.Icon ) );
    ptPicture: SetOrdProp( GetComponent( 0 ), GetPropInfo, LongInt( Picture ) );
    end;
  end
else
  begin
  if frmPictPlus.Cleared then
    begin
    FrontLoad := TPicture.Create;
    case frmPictPlus.PropertyType of
      ptBitmap: SetOrdProp( GetComponent( 0 ), GetPropInfo, LongInt( FrontLoad.Bitmap ) );
      ptIcon: SetOrdProp( GetComponent( 0 ), GetPropInfo, LongInt( FrontLoad.Icon ) );
      ptPicture: SetOrdProp( GetComponent( 0 ), GetPropInfo, LongInt( FrontLoad ) );
      end;
    FrontLoad.Free;
    end;
  end;
Picture.Free;
frmPictPlus.Free;
end;

procedure TPicturePropertyPlus.SetFileType( const Filename: String );
begin
end;

{ ******************************************************************* }
function TPicturePropertyPlus.GetValue: String;
var
  Picture: TPicture;
  EnumType: PTypeInfo;
begin
Result := '(None)';
EnumType := GetPropType;
if UpperCase( ENumType.Name ) = 'TBITMAP' then
  begin
  if not TBitmap( GetOrdProp( GetComponent( 0 ), GetPropInfo ) ).Empty then
    Result := '(TBitMap)'
  end
else if UpperCase( ENumType.Name ) = 'TICON' then
  begin
  if not TIcon( GetOrdProp( GetComponent( 0 ), GetPropInfo ) ).Empty then
    Result := '(TIcon)'
  end
else if UpperCase( ENumType.Name ) = 'TPICTURE' then
  begin
  Picture := TPicture.Create;
  Picture.Assign( TPicture( GetOrdProp( GetComponent( 0 ), GetPropInfo ) ) );
  if Picture.Height = 0 then
    begin
    Picture.Free;
    exit;
    end;
  if not Picture.Bitmap.Empty then
    begin
    Result := '(TBitMap)';
    Picture.Free;
    exit;
    end;
  Picture.Assign( TPicture( GetOrdProp( GetComponent( 0 ), GetPropInfo ) ) );
  if not Picture.Icon.Empty then
    begin
    Result := '(TIcon)';
    Picture.Free;
    exit;
    end;
  Picture.Assign( TPicture( GetOrdProp( GetComponent( 0 ), GetPropInfo ) ) );
  if not Picture.Metafile.Empty then
    begin
    Result := '(TMetaFile)';
    Picture.Free;
    exit;
    end;
  Picture.Free;
  end
end;

procedure TPicturePropertyPlus.SetValue( const Value:String );
begin
end;

function  TPicturePropertyPlus.GetAttributes: TPropertyAttributes;
begin
Result := [ paDialog ];
end;

procedure Register;
begin
RegisterPropertyEditor( TypeInfo( TBitMap ), nil, '', TPicturePropertyPlus );
RegisterPropertyEditor( TypeInfo( TIcon ), nil, '', TPicturePropertyPlus );
//RegisterPropertyEditor( TypeInfo( TPicture ), nil, '', TPicturePropertyPlus );
end;


procedure TfrmPictPlus.cbxFileClick(Sender: TObject);
begin
btnOk.Enabled := True;
imgView.Picture.LoadFromFile( cbxFile.Filename );
//SetFileType( cbxFile.Filename );
if PropertyType = ptBitmap then
  begin
  btnTest.Glyph := imgView.Picture.Bitmap;
  end;
end;

procedure TfrmPictPlus.cbxFileDblClick(Sender: TObject);
begin
if cbxFile.Items.Count > 0 then
  begin
  btnOK.Enabled := True;
  btnOK.Click;
  end;
end;

procedure TfrmPictPlus.cbxHistoryClick(Sender: TObject);
begin
if cbxDir.Directory <> cbxHistory.Items[ cbxHistory.ItemIndex ] then
  begin
  cbxDir.Directory := cbxHistory.Items[ cbxHistory.ItemIndex ];
  btnOK.Enabled := False;
  end;
cbxHistory.Width := OldHistoryWidth;
end;

procedure TfrmPictPlus.FormCreate(Sender: TObject);
var
  History: Integer;
begin
Ini := TRegIniFile.Create( '\Software\TTI Consulting\Components\PicturePlus' );
OldHistoryWidth := cbxHistory.Width;
{ Load History }
cbxDir.Directory := Ini.ReadString( 'Directory History', 'Last Directory',
  cbxDir.Directory );
History := 0;
cbxHistory.Items.Clear;
while Length( Ini.ReadString( 'Directory History', 'Dir' + IntToStr( History ), '' ) ) > 0 do
  begin
  cbxHistory.Items.Add( Ini.ReadString( 'Directory History', 'Dir' + IntToStr( History ), '' ) );
  Inc( History );
  if History = 15 then
    exit;
  end;
end;

procedure TfrmPictPlus.cbxHistoryExit(Sender: TObject);
begin
cbxHistory.Width := OldHistoryWidth;
end;

procedure TfrmPictPlus.cbxHistoryDropDown(Sender: TObject);
begin
cbxHistory.BringToFront;
cbxHistory.Width := cbxHistory.Left + pnlBottom.Width - 20;
end;

procedure TfrmPictPlus.cbxDirChange(Sender: TObject);
begin
btnOK.Enabled := False;
if lsvImages.Visible then
  LoadListView;
end;

procedure TfrmPictPlus.btnClearClick(Sender: TObject);
begin
Cleared := True;
Close;
end;

procedure TfrmPictPlus.imgViewClick(Sender: TObject);
begin
imgView.Stretch := not imgView.Stretch;
end;

procedure TfrmPictPlus.btnOKClick(Sender: TObject);
begin
if cbxHistory.Items.IndexOf( cbxDir.Directory ) = -1 then
  cbxHistory.Items.Insert( 0, cbxDir.Directory );
end;

procedure TfrmPictPlus.FormClose(Sender: TObject;
  var Action: TCloseAction);
var
  History: Integer;
begin
for History := 0 to cbxHistory.Items.Count - 1 do
  Ini.WriteString( 'Directory History', 'Dir' + IntToStr( History ), cbxHistory.Items[ History ] );
Ini.WriteString( 'Directory History', 'Last Directory', cbxDir.Directory );
Ini.WriteInteger( 'Window Position', 'Height', Height );
Ini.WriteInteger( 'Window Position', 'Width', Width );
Ini.WriteInteger( 'Window Position', 'Top', Top );
Ini.WriteInteger( 'Window Position', 'Left', Left );
end;

procedure TfrmPictPlus.chbEnableDisableClick(Sender: TObject);
begin
btnTest.Enabled := not btnTest.Enabled;
end;

procedure TfrmPictPlus.btnAboutClick(Sender: TObject);
begin
pnlAbout.Visible := not pnlAbout.Visible;
end;

procedure TfrmPictPlus.LoadListView;
var
  Cnt: Integer;
  PictureCnt: Integer;
  NewItem: TListItem;
  Picture: TPicture;
begin
if lsvImages.Visible then
  begin
  Screen.Cursor := crHourGlass;
  iltImages.Clear;
  lsvImages.Items.Clear;
  Picture := TPicture.Create;
  PictureCnt := 0;
  for Cnt := 0 to cbxFile.Items.Count - 1 do
    begin
    NewItem := lsvImages.Items.Add;
    NewItem.Caption := cbxFile.Items[ Cnt ];
    if Pos( '.BMP', UpperCase( cbxFile.Items[ Cnt ] ) ) > 0 then
      begin
      Inc( PictureCnt );
      iltImages.FileLoad( rtBitMap, cbxDir.Directory + '/' + cbxFile.Items[ Cnt ], clMaroon );
      NewItem.ImageIndex := Cnt;
      end
    else if Pos( '.ICO', UpperCase( cbxFile.Items[ Cnt ] ) ) > 0 then
      begin
      Inc( PictureCnt );
      iltImages.FileLoad( rtIcon, cbxDir.Directory + '/' + cbxFile.Items[ Cnt ], clMaroon );
      NewItem.ImageIndex := Cnt;
      end
    else
      MessageDlg( 'Unknown: ' + cbxFile.Items[ Cnt ], mtError, [ mbOK ], 0 );
    end;
  Picture.Free;
  Screen.Cursor := crDefault;
  end;
end;

procedure TfrmPictPlus.btnViewClick(Sender: TObject);
begin
if lsvImages.Visible then
  begin
  lsvImages.Visible := False;
  cbxFile.Visible := True;  
  end
else
  begin
  lsvImages.Visible := True;
  cbxFile.Visible := False;
  LoadListView;
  end;
end;

procedure TfrmPictPlus.lsvImagesClick(Sender: TObject);
begin
if lsvImages.Selected <> nil then
  begin
  btnTest.Glyph.LoadFromFile( cbxDir.Directory + '/' + lsvImages.Selected.Caption );
  btnOK.Enabled := True;
  end
else
  btnOK.Enabled := False;
end;

procedure TfrmPictPlus.lsvImagesDblClick(Sender: TObject);
begin
if lsvImages.Selected <> nil then
  begin
  btnTest.Glyph.LoadFromFile( cbxDir.Directory + '/' + lsvImages.Selected.Caption );
  btnOK.Enabled := True;
  btnOK.Click;
  end
end;


end.
